@@ -31,6 +31,13 @@ struct _EogImageListPrivate {
 
 	/* Number of images. Same as g_list_length (store) */
 	int             n_images;
+
+	/* number of images to load before emitting a 'prepared' signal */
+	int             load_n_files;
+
+	/* the largest common part of all image uris */
+	GnomeVFSURI     *base_uri;
+	gboolean         no_common_base;
 };
 
 typedef struct {
@@ -49,6 +56,7 @@ enum {
 };
 static guint eog_image_list_signals[LAST_SIGNAL];
 
+static GSList *supported_mime_types = NULL;
 
 /* ====================================================== */
 /*               Cache Area                               */
@@ -117,6 +125,11 @@ eog_image_list_dispose (GObject *object)
 		priv->age++;
 		priv->n_images = 0;
 	}
+
+	if (priv->base_uri != NULL) {
+		gnome_vfs_uri_unref (priv->base_uri);
+		priv->base_uri = NULL;
+	}
 }
 
 static void
@@ -131,6 +144,8 @@ eog_image_list_instance_init (EogImageList *obj)
 	priv->age = 0;
 	priv->store = NULL;
 	priv->n_images = 0;
+	priv->base_uri = NULL;
+	priv->no_common_base = FALSE;
 }
 
 static void 
@@ -223,7 +238,8 @@ free_loading_context (DirLoadingContext *ctx)
  * to the filename. Otherwise the image is prepended to the list.
  */
 static void 
-add_image_private (EogImageList *list, EogImage *image, gboolean sorted) { 
+add_image_private (EogImageList *list, EogImage *image, gboolean sorted) 
+{ 
 	EogImageListPrivate *priv;
 
 	g_return_if_fail (EOG_IS_IMAGE_LIST (list));	
@@ -245,6 +261,65 @@ add_image_private (EogImageList *list, EogImage *image, gboolean sorted) {
 	priv->n_images++;
 }
 
+static gint
+compare_quarks (gconstpointer a, gconstpointer b)
+{
+	return GPOINTER_TO_INT (a) - GPOINTER_TO_INT (b);
+}
+
+/* Creates a list of Quarks which represent all mime types supported
+ * by GdkPixbuf at the moment */
+static GSList*
+get_supported_mime_types (void)
+{
+	GSList *format_list;
+	GSList *it;
+	GSList *list = NULL;
+	gchar **mime_types;
+	int i;
+	
+	format_list = gdk_pixbuf_get_formats ();
+
+	for (it = format_list; it != NULL; it = it->next) {
+		mime_types = gdk_pixbuf_format_get_mime_types ((GdkPixbufFormat *) it->data);
+
+		for (i = 0; mime_types[i] != NULL; i++) {
+			GQuark quark;
+
+			quark = g_quark_from_string (mime_types[i]);
+			list = g_slist_prepend (list,
+						GINT_TO_POINTER (quark));
+		}
+
+		g_strfreev (mime_types);
+	}
+
+	list = g_slist_sort (list, (GCompareFunc) compare_quarks);
+	
+	g_slist_free (format_list);
+
+	return list;
+}
+
+/* checks if the mime type is in our static list of 
+ * supported mime types.
+ */
+static gboolean 
+is_supported_mime_type (const char *mime_type) 
+{
+	GQuark quark;
+	GSList *result;
+
+	if (supported_mime_types == NULL) {
+		supported_mime_types = get_supported_mime_types ();
+	}
+
+	quark = g_quark_from_string (mime_type);
+	
+	result = g_slist_find (supported_mime_types, GINT_TO_POINTER (quark));
+
+	return (result != NULL);
+}
 
 /* ================== Directory Loading stuff ===================*/
 
@@ -271,7 +346,7 @@ directory_visit_cb (const gchar *rel_uri,
 	priv = list->priv;
 
 	if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) > 0) {
-		if (g_strncasecmp (info->mime_type, "image/", 6) == 0) {
+		if (is_supported_mime_type (info->mime_type)) {
 			load_uri = TRUE;
 		}
 	}
@@ -308,6 +383,10 @@ real_dir_loading (DirLoadingContext *ctx)
 	list = ctx->list;
 	priv = list->priv;
 
+	g_assert (ctx->uri != NULL);
+
+	priv->base_uri = gnome_vfs_uri_dup (ctx->uri);
+
 	gnome_vfs_directory_visit_uri (ctx->uri,
 				       GNOME_VFS_FILE_INFO_DEFAULT |
 				       GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
@@ -325,13 +404,76 @@ real_dir_loading (DirLoadingContext *ctx)
 	
 	return FALSE;
 }
+
+/* Delayed file loading function, called when idle. Loads an file
+ * pointed by ctx->uri.  
+ */
+static gint
+real_file_loading (DirLoadingContext *ctx)
+{
+	EogImageList *list;
+	EogImageListPrivate *priv;
+	EogImage *image = NULL;
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult result;
+
+	info = ctx->info;
+
+	g_return_val_if_fail (info->type == GNOME_VFS_FILE_TYPE_REGULAR, FALSE);
+
+
+	result = gnome_vfs_get_file_info_uri (ctx->uri, info,
+					      GNOME_VFS_FILE_INFO_DEFAULT |
+					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
+					      GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
+	if (result != GNOME_VFS_OK) {
+		return FALSE;
+	}
+
+	list = ctx->list;
+	priv = list->priv;
+
+	if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) > 0 &&
+	    is_supported_mime_type (info->mime_type)) 
+	{
+		image = eog_image_new_uri (ctx->uri);
+	}
+
+	if (image != NULL) {
+		add_image_private (list, image, TRUE);
+		g_object_unref (image);
+
+		/* Retrieve base uri. This may not possible if there
+		 * are a lot of single images from different
+		 * directories. In this case base_uri is set to NULL
+		 * and no_common_base is TRUE.
+		 */
+		if ((priv->base_uri == NULL) && !priv->no_common_base) {
+			priv->base_uri = gnome_vfs_uri_get_parent (ctx->uri);
+		}
+		else if ((priv->base_uri != NULL) && !gnome_vfs_uri_is_parent (priv->base_uri, ctx->uri, FALSE)) {
+			gnome_vfs_uri_unref (priv->base_uri);
+			priv->base_uri = NULL;
+			priv->no_common_base = TRUE;
+		}
+	}
+		
+	priv->load_n_files--;
+
+	if (priv->load_n_files == 0) {
+		g_signal_emit (G_OBJECT (list), eog_image_list_signals[LIST_PREPARED], 0);
+	}
+			
+       	free_loading_context (ctx);
+
+	return FALSE;
+}
 	
 /* Helper function to initialize the DirLoadingContext structure */
 static DirLoadingContext*
 prepare_loading_context (EogImageList *list, const gchar *text_uri) 
 {
 	DirLoadingContext *ctx;
-	GnomeVFSFileInfo *info;
 	GnomeVFSResult result;
 
 	ctx = g_new0 (DirLoadingContext, 1);
@@ -341,8 +483,8 @@ prepare_loading_context (EogImageList *list, const gchar *text_uri)
 	g_message ("Prepare context for URI: %s", text_uri);
 #endif
 
-	info = gnome_vfs_file_info_new ();
-	result = gnome_vfs_get_file_info_uri (ctx->uri, info,
+	ctx->info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info_uri (ctx->uri, ctx->info,
 					      GNOME_VFS_FILE_INFO_DEFAULT |
 					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 	if (result != GNOME_VFS_OK) {
@@ -350,7 +492,6 @@ prepare_loading_context (EogImageList *list, const gchar *text_uri)
 		return NULL;
 	}
 
-	ctx->info = info;
 	ctx->handle = 0;
 	ctx->list = list;
 
@@ -373,25 +514,47 @@ eog_image_list_add_directory (EogImageList *list, char *text_uri)
 
 	ctx = prepare_loading_context (list, text_uri);
 	
-	if (ctx != NULL) {
-		if (ctx->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
-			g_idle_add ((GSourceFunc)real_dir_loading, ctx);
-#if 0
-		/* FIXME: do we want to support single files? */
-		else if (ctx->info->type == GNOME_VFS_FILE_TYPE_REGULAR)
-			g_idle_add ((GtkFunction) real_file_loading, ctx);
-#endif
-		else {
+	if ((ctx != NULL) && (ctx->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)) {
+		g_idle_add ((GSourceFunc)real_dir_loading, ctx);
+	}
+	else {
+		if (ctx != NULL) {
 			free_loading_context (ctx);
-			g_warning ("Can't handle URI: %s", text_uri);
-			return;
 		}
-	} else {
 		g_warning ("Can't handle URI: %s", text_uri);
-		return;
 	}	
 }
 
+void
+eog_image_list_add_files (EogImageList *list, GList *uri_list) 
+{
+	GList *it;
+	EogImageListPrivate *priv;
+	DirLoadingContext *ctx;
+
+	priv = list->priv;
+
+	if (uri_list == NULL) {
+		return;
+	}
+
+	priv->load_n_files = g_list_length (uri_list);
+
+	for (it = uri_list; it != NULL; it = it->next) {
+		ctx = prepare_loading_context (list, (char*) it->data);
+
+		if ((ctx != NULL) && (ctx->info->type == GNOME_VFS_FILE_TYPE_REGULAR)) {
+			g_idle_add ((GSourceFunc) real_file_loading, ctx);
+		}
+		else {
+			if (ctx != NULL) {
+				free_loading_context (ctx);
+			}
+			priv->load_n_files--;
+		}
+	}
+}
+
 /* ====================================================== */
 /*               Public API                               */
 
@@ -447,6 +610,18 @@ eog_image_list_length (EogImageList *list)
 	return list->priv->n_images;
 }
 
+GnomeVFSURI*
+eog_image_list_get_base_uri (EogImageList *list)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE_LIST (list), NULL);
+	
+	if (list->priv->base_uri != NULL)
+		gnome_vfs_uri_ref (list->priv->base_uri);
+
+	return list->priv->base_uri;
+}
+
+
 void
 eog_image_list_add_image (EogImageList *list, EogImage *image)
 {
@@ -726,3 +901,4 @@ eog_image_list_iter_equal (EogImageList *list, EogIter *a, EogIter *b)
 	
 	return (a->list == b->list) && (a->age == b->age) && (a->position == b->position);
 }
+
