@@ -33,6 +33,7 @@
 #include <bonobo/bonobo-macros.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <eel/eel-vfs-extensions.h>
 
 #include "eog-image-view.h"
@@ -116,6 +117,7 @@ enum {
 static void popup_menu_cb (gpointer data, guint action, GtkWidget *widget);
 static gint save_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 			 CORBA_Environment *ev, void *closure);
+static gboolean save_uri (EogImageView *view, const char *text_uri, GdkPixbufFormat *format);
 
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -184,6 +186,8 @@ verb_FullScreen_cb (BonoboUIComponent *uic, gpointer data, const char *name)
 	list = eog_image_list_new ();
 	eog_image_list_add_image (list, EOG_IMAGE (image_view->priv->image));
 	fs = eog_full_screen_new (list, NULL);
+	g_signal_connect (G_OBJECT (fs), "hide", G_CALLBACK (gtk_widget_destroy), NULL);
+
 	g_object_unref (list);
 
 	gtk_widget_show_all (fs);
@@ -313,6 +317,7 @@ verb_SaveAs_cb (BonoboUIComponent *uic, gpointer user_data,
 	GtkWidget        *dlg;
 	int              response;
 	gchar            *filename = NULL;
+	GdkPixbufFormat  *format = NULL;
 
 	g_return_if_fail (EOG_IS_IMAGE_VIEW (user_data));
 
@@ -320,19 +325,20 @@ verb_SaveAs_cb (BonoboUIComponent *uic, gpointer user_data,
 
 	if (image_view->priv->image == NULL) return;
 
-	dlg = eog_file_selection_new (EOG_FILE_SELECTION_SAVE);
+	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_SAVE);
 	gtk_widget_show_all (dlg);
 
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
 
 	if (response == GTK_RESPONSE_OK) {
-		filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dlg)));
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
+		format = eog_file_selection_get_format (EOG_FILE_SELECTION (dlg));
 	}
 
 	gtk_widget_destroy (dlg);
 
 	if (response == GTK_RESPONSE_OK) {
-		save_uri_cb (NULL, filename, NULL, image_view);
+		save_uri (image_view, filename, format);
 	}
 
 	if (filename != NULL) {
@@ -498,19 +504,34 @@ eog_image_view_get_prop (BonoboPropertyBag *bag,
 		break;
 	}
 	case PROP_WINDOW_TITLE: {
+		char *caption;
+
 		g_assert (arg->_type == BONOBO_ARG_STRING);
 
-		if (priv->image != NULL) {
-			BONOBO_ARG_SET_STRING (arg, eog_image_get_caption (priv->image));
+		caption = g_strdup (eog_image_get_caption (priv->image)); 
+
+		if (eog_image_is_modified (priv->image)) {
+			char *tmp;
+
+			tmp = g_strconcat (caption, "*", NULL);
+			g_free (caption);
+			caption = tmp;
 		}
 
+		if (priv->image != NULL) {
+			BONOBO_ARG_SET_STRING (arg, caption);
+		}
+		g_free (caption);
+
 		break;
 	}
 	case PROP_WINDOW_STATUS: {
 		gchar *text = NULL;
+		gchar *size_string;
 		int zoom = 0;
 		int width = 0;
 		int height = 0;
+		GnomeVFSFileSize bytes = 0;
 
 		g_assert (arg->_type == BONOBO_ARG_STRING);
 
@@ -518,12 +539,17 @@ eog_image_view_get_prop (BonoboPropertyBag *bag,
 
 		if (priv->image != NULL) {
 			eog_image_get_size (priv->image, &width, &height);
+			bytes = eog_image_get_bytes (priv->image);
 		}
 		
 		if ((width > 0) && (height > 0)) {
-			/* [image width] x [image height] pixel  [zoom in percent] */
-			text = g_strdup_printf (_("%i x %i pixel    %i%%"), 
-						width, height, zoom);
+			size_string = gnome_vfs_format_file_size_for_display (bytes);
+
+			/* [image width] x [image height] pixel  [bytes]    [zoom in percent] */
+			text = g_strdup_printf (_("%i x %i pixel  %s    %i%%"), 
+						width, height, size_string, zoom);
+
+			g_free (size_string);
 		}
 		else {
 			text = g_strdup (" ");
@@ -965,6 +991,14 @@ image_changed_cb (EogImage *img, gpointer data)
 					      "window/status",
 					      arg, NULL);
 	bonobo_arg_release (arg);
+
+	arg = bonobo_arg_new (BONOBO_ARG_STRING);
+	eog_image_view_get_prop (NULL, arg, PROP_WINDOW_TITLE, NULL, view);
+
+	bonobo_event_source_notify_listeners (view->priv->property_bag->es,
+					      "window/title",
+					      arg, NULL);
+	bonobo_arg_release (arg);
 }
 
 static void
@@ -1069,30 +1103,44 @@ static gint
 save_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 	     CORBA_Environment *ev, void *closure)
 {
-	EogImageView *view;
-	GnomeVFSURI *uri;
+	return save_uri (EOG_IMAGE_VIEW (closure), text_uri, NULL);
+}
+
+static gboolean
+save_uri (EogImageView *view, const char *text_uri, GdkPixbufFormat *format)
+{
 	GError *error = NULL;
 	GtkWidget *dialog;
-	gboolean result;
-
-	g_return_val_if_fail (EOG_IS_IMAGE_VIEW (closure), 1);
-	g_return_val_if_fail (text_uri != NULL, 1);
-
-	view = EOG_IMAGE_VIEW (closure);
+	gboolean result = FALSE;
+	
+	g_return_val_if_fail (EOG_IS_IMAGE_VIEW (view), FALSE);
+	
 	if (view->priv->image == NULL) return FALSE;
+	
+	if (text_uri != NULL) {
+		GnomeVFSURI *uri;
+		
+                /* FIXME: what kind of escaping do we need here? */
+		uri = gnome_vfs_uri_new (text_uri);
+		
+		result = eog_image_save (view->priv->image, uri, format, &error);
 
-	/* FIXME: what kind of escaping do we need here? */
-	uri = gnome_vfs_uri_new (text_uri);
-
-	result = eog_image_save (view->priv->image, uri, &error);
-
+		gnome_vfs_uri_unref (uri);
+	}
+		
 	if (result) {
 		dialog = eog_hig_dialog_new (GTK_STOCK_DIALOG_INFO,
 					     _("Image successfully saved"), NULL, FALSE);
 	}
 	else {
+		char *message = NULL;
+
+		if (error != NULL) {
+			message = error->message;
+		}
+
 		dialog = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR,
-					     _("Image saving failed"), error->message, FALSE);
+					     _("Image saving failed"), message, FALSE);
 	}
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
 	g_signal_connect_swapped (dialog, "response",
